# Andrey version 1.0.0
# AndreyInterpreter.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of Andrey, a pedagogical implementation of Markov chains.
# Andrey is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Andrey is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Andrey.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Scripts

# 1) Imported modules
# -------------------

import string
import sys
import re
import math

from S2MLMatrices import *
from S2MLInterpreter import *

# 2) Creator
# ----------

class AndreyInterpreter(S2MLInterpreter):
  def __init__(self, engine):
    S2MLInterpreter.__init__(self, engine)

# 3) Scripts
# ----------

  def _ParseCommand(self):
    commandName = self.LookUpArgument()
    if commandName=="assess":
      commandName = self.GetArgument()
      self._errors += self._ParseCommandAssess()
    else:
      S2MLInterpreter._ParseCommand(self)

  def _ParseCommandAssess(self):
    what = self.GetArgument()
    if what==None:
      self._RaiseError("line %d\nDTMC/CTMC was expected\n" % self._lineNumber)
      return 1
    if what=="DTMC":
      return self._ParseCommandAssessDTMC()
    if what=="CTMC":
      return self._ParseCommandAssessCTMC()
    self._RaiseError("line %d\nDTMC/CTMC was expected\n" % self._lineNumber)
    return 1

  # assess DTMC blockName (numberOfIterations | numbersOfIterations) [probabilities=bool] [point-rewards=bool] fileName [mode=(append|write)]
  def _ParseCommandAssessDTMC(self):
    blockName = self.GetArgument()
    if blockName==None:
      self._RaiseError("line %d\nA block name was expected\n" % self._lineNumber)
      return 1
    numbersOfIterations = self.GetIntList()
    if numbersOfIterations==None:
      return 1
    probabilities = self.GetBoolOption("probabilities", False)
    if probabilities==None:
      return 1
    pointRewards = self.GetBoolOption("point-rewards", False)
    if pointRewards==None:
      return 1
    fileName = self.GetArgument()
    if fileName==None:
      self._RaiseError("line %d\nA file name was expected\n" % self._lineNumber)
      return 1
    mode = self.GetModeOption("w")
    if mode==None:
      return 1
    return self._engine.AssessDiscreteTimeMarkovSystem(blockName, numbersOfIterations, probabilities, pointRewards, fileName, mode);

  # assess CTMC blockName (mission-time | missionTimes) [probabilities=bool] [soujourn-times=bool] [point-rewards=bool] [mean-rewards=bool] fileName [mode=(append|write)]
  def _ParseCommandAssessCTMC(self):
    blockName = self.GetArgument()
    if blockName==None:
      self._RaiseError("line %d\nA block name was expected\n" % self._lineNumber)
      return 1
    missionTimes = self.GetFloatList()
    if missionTimes==None:
      return 1
    probabilities = self.GetBoolOption("probabilities", False)
    if probabilities==None:
      return 1
    sojournTimes = self.GetBoolOption("sojourn-times", False)
    if sojournTimes==None:
      return 1
    pointRewards = self.GetBoolOption("point-rewards", False)
    if pointRewards==None:
      return 1
    meanRewards = self.GetBoolOption("mean-rewards", False)
    if meanRewards==None:
      return 1
    fileName = self.GetArgument()
    if fileName==None:
      self._RaiseError("line %d\nA file name was expected\n" % self._lineNumber)
      return 1
    mode = self.GetModeOption("w")
    if mode==None:
      return 1
    return self._engine.AssessContinuousTimeMarkovSystem(blockName, missionTimes, probabilities, sojournTimes, pointRewards, meanRewards, fileName, mode);
